#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Jul  3 20:00:23 2017

@author: PhilKirk
"""

import glob
import pandas
import numpy
from matplotlib import cm
from matplotlib import pyplot as plt
from pathlib import Path
import numpy as np


#Use only forward slashes in the directory path strings
in_directorys = [r"C:/Users/Phil/OneDrive - University of Leeds/PhD/March 2020 Important Y and S drive items/Y Drive/2019/191024 psuc2 svl1 ats s 4 dpg/analysis/TZ/01N 720",
                 r"C:/Users/Phil/OneDrive - University of Leeds/PhD/March 2020 Important Y and S drive items/Y Drive/2019/191024 psuc2 svl1 ats s 4 dpg/analysis/TZ/01N svl1"]

sample_names = ['pSUC2:GFP', 'pSUC2:GFP x $svl1$']

rescale_y_axis = False #for none normalised

def concatenate(directory, normY = False, groupX = True, normX = True): 
    relevant_files = directory / "*.csv"
    filelist=glob.glob(str(relevant_files))
    dfList=[]
    for filename in filelist:
        print(filename)
        df=pandas.read_csv(filename, header = 0)
        print("Number of columns = " + str(len(df.columns)))
        if len(df.columns) == 3:
            df.columns=['A','B','C']
            df = df.drop('A', 1)
        df2 = normalise(df, normY = normY, groupX = groupX, normX = normX)
        df2["ID"] = filename
        dfList.append(df2) 
        concatDf=pandas.concat(dfList, axis= 0)
    return concatDf

def normalise(df, normX = True, normY = False, groupX = True, foldRoot = False): #Set normX to false if cut off is being made eg from root tip to some point
    df.columns=["X","Y"]
    if normX == True:
        df["X"] -= df["X"].min()
        df["X"] /= df["X"].max()
        df.X = df.X.round(decimals=2)
        if foldRoot == True:
            binX = []
            for row in df['X']:
                if row < 0.5:
                    binX.append(1-row)
                else:
                    binX.append(row)
            df['X'] = binX
        df['X'] = (df['X']*2)-1
        df.X = df.X.round(decimals=2)
    if normY == True:
        df["Y"] /= df["Y"].max()
    if groupX == True:
        df2 = df.groupby(["X"])["Y"].mean()
        df2 = df2.to_frame()
    else:
        df2 = df
    return df2
       
def summarise(concatDf):
    concatDf.reset_index(level=0, inplace=True)
    concatDf2a = concatDf.groupby(["X"])["Y"].mean()
    concatDf2b = concatDf.groupby(["X"])["Y"].std()
    concatDf2c = concatDf.groupby(["X"])["Y"].count()
    concatDf2d = numpy.sqrt(concatDf2c)
    concatDf2 = pandas.concat([concatDf2a, concatDf2b, concatDf2c, concatDf2d], axis=1)
    concatDf2.reset_index(level=0, inplace=True)
    concatDf2.columns = ['X','Y','SD','N','sqR-N']
    concatDf2['SE'] = concatDf2['SD']/concatDf2['sqR-N'].fillna(0)
    concatDf2['offsetX'] = concatDf2["X"] - (concatDf2["X"].max()/2)
    concatDf2['YminSE'] = concatDf2['Y'] - concatDf2['SE']
    concatDf2['YpluSE'] = concatDf2['Y'] + concatDf2['SE']
    cols = ['sqR-N', 'N', 'SD', 'X', 'YminSE', 'YpluSE', 'offsetX', 'Y', 'SE']
    concatDf2 = concatDf2[cols]
    return(concatDf2)

def plot_trace(*args, normX = True, rescale_y_axis = False, color_pallete = 'Set1', fig_name = 'fig', directory = Path.cwd()):
    if type(color_pallete).__name__ == 'ListedColormap':
        cmap = color_pallete
    else: cmap = cm.get_cmap(color_pallete)
    colour_index = 0
    plt.rcParams['figure.constrained_layout.use'] = True
    plt.rc('font', size=36)          # controls default text sizes
    plt.rc('axes', titlesize=36)     # fontsize of the axes title
    plt.rc('axes', labelsize=36)     # fontsize of the x and y labels
    plt.rc('xtick', labelsize=42)    # fontsize of the tick labels
    plt.rc('ytick', labelsize=42)    # fontsize of the tick labels
    plt.rc('legend', fontsize=36)    # legend fontsize
    
    fig, ax = plt.subplots(1, 1, figsize=(10.5,13.5))
    for i, dataf in enumerate(args):
        dataf.loc[:,'X'] = ((dataf.loc[:,'X']+1) /2)*150 #last number: X tick max
        plt.plot(dataf.X, dataf.Y, c = cmap(colour_index), label = sample_names[i], linewidth=6) # this plots a line with colour k
        plt.fill_between(dataf.X, dataf.Y, dataf.YminSE, color= cmap(colour_index), alpha=0.2)
        plt.fill_between(dataf.X, dataf.Y, dataf.YpluSE, color= cmap(colour_index), alpha=0.2)
        colour_index += 1/len(cmap.colors)
    ax.spines['right'].set_visible(False)
    ax.spines['top'].set_visible(False)
    ax.spines['bottom'].set_linewidth(3.6)
    ax.spines['left'].set_linewidth(3.6)
    ax.spines['right'].set_linewidth(3.6)
    ax.spines['top'].set_linewidth(3.6)
    
    plt.ylabel("Mean grey value (AU)")
    
    ax.get_xaxis().set_ticks([])
    plt.xlabel(r'$\longleftarrow$ Position along transect $\longrightarrow$')
    
    #plt.xlabel(r'TZ/PZ $\to$ tip (μm)')
    
    if rescale_y_axis:
        fig_name1 = fig_name + '.png'
        path1 = directory / fig_name1
        fig.savefig(path1)
        labelsY = ax.get_yticklabels(which='both')
        locs, labels = plt.yticks()            # Get locations and labels
        #print(labelsY)
        print(locs)
        for lab in labelsY:
            print(lab)
        plt.yticks(np.arange(0, max(locs), step=max(locs)/5))
        #print(numpy.linspace(0,100,len(locs)))
        plt.yticks(np.arange(0, max(locs), step=max(locs)/5), numpy.linspace(0,100,5))  # Set locations and labels
    
    handles, labels = ax.axes.get_legend_handles_labels()
    labels = sample_names
    # TODO Legend in broken
    plt.legend(handles, labels, bbox_to_anchor=(0.5, 0.98), loc= 8, borderaxespad=0., edgecolor = 'w')
    fig_name = fig_name + '.png'
    path = directory / fig_name
    #fig.show()
    fig.savefig(path)



def main(groupX = True, normY = False, normX = True, rescale_y_axis = rescale_y_axis, color_pallete = 'tab20', fig_name = 'fig'):
    summary_list = []  
    for indir, sample_name in zip(in_directorys, sample_names): 
        indir = Path(indir)
        print(indir)
        treatment = concatenate(indir, groupX = groupX, normY = normY,normX = normX)
        Summary = summarise(treatment)
        summary_list.append(Summary)
    plot_trace(*summary_list, normX = normX, color_pallete = color_pallete,rescale_y_axis = rescale_y_axis, fig_name = fig_name, directory = Path(in_directorys[1]).parent)
    return summary_list

N = 256
vals = np.ones((N, 4))


custom_pallete = 'Set1'
keepcharacters = (' ','.','_')
file_safe_name = "".join(c for c in sample_names[-1] if c.isalnum() or c in keepcharacters).rstrip()
no_norm_summary = main(groupX = True, normY = False,rescale_y_axis = rescale_y_axis, fig_name = 'no normY ' +file_safe_name, color_pallete = custom_pallete)
norm_summary =main(groupX = True, normY = True, rescale_y_axis = False, fig_name = 'normY '+  file_safe_name, color_pallete = custom_pallete)

for sample, no_norm_df in zip(sample_names, no_norm_summary):
    dir_export = Path(in_directorys[1])
    csv_filename = sample + " no norm.csv"
    file_safe_name = "".join(c for c in csv_filename if c.isalnum() or c in keepcharacters).rstrip()
    no_norm_df.to_csv(dir_export.parent / file_safe_name)










