#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Runs RootNav .csv output to extract per plant root length and root count data
"""

import pandas as pd
from pathlib import PurePath
#whichfile = input('Which file? (example.csv): ')
pixpermm = 2311 / 120 #for 12cm square plates scanned at 500 dpi

print(pixpermm, ' pixels per mm')
csv_filepath = PurePath('C:/Users/Phil/OneDrive - University of Leeds/PhD/March 2020 Important Y and S drive items/Y Drive/2019/191107 8dpg svl1 s ns ats/analysis/Root measurements.csv')

#set .csv and pixels per mm
data = pd.read_csv(csv_filepath)
if data.columns[1] == 'Total Length':
    data['Total Length'] = data['Total Length'].astype(float)
    data['Total Length'] = data['Total Length'] / pixpermm
    
if data.columns[1] == 'Tortuosity':
    data['Total Length'] = data['Tortuosity'].astype(float)

    
#split data by tags
data1 = data['Tag'].str.split(pat='_', n=-1, expand=True)
tagcolumns = list(data1)  # list of tags, [:-1] are tags holding conditions or even dates
rootstring = tagcolumns[-1:]#the last column of tags is needed to define idividual plants and root type

#replace tags

my_dict = {"p35S::SVL1-YFP": ["YB10"], "WT" : ["wt"], "0.1 mM nitrate":["01mMN"],"9 mM nitrate":["9mMN"],
           "1 mM nitrate":["1mMN"],"1% sucrose":["s"], "0% sucrose":["ns"], "Control":["con"], "3% PEG": ["peg"]}

key_list = []
val_list = []

for key in my_dict:
    for val in my_dict[key]:
        key_list.append(key)
        val_list.append(val)
  
for col in tagcolumns[:-1]:
    for row in data1.index:
        val = data1.loc[row,col]
        if val in val_list:
            key = key_list[val_list.index(val)]
            data1.loc[row,col] = key

#build plant ID
data2 = pd.concat([data1, data], axis = 1)
s = data2[rootstring[0]].apply(lambda x: x.split(':'))
data2['left'] = s.apply(lambda x: x[0])
data2['right'] = s.apply(lambda x: x[1])
data2['left'] = data2['left'].astype(float)
data2['left'] *= 100 

#build root type and summarise data by plant 
data3 = data2['right'].str.split(pat='.', n=-1, expand=True)
data3list = list(data3)
if len(data3list) == 2:
    data3.columns = ['right3', 'Last']
    data3['Last'] = 'A'
else:
    data3.columns = ['right3','1', 'Last']
    data3['Last'][data3['Last'].notnull()] = 'C'
    data3['Last'].fillna(value='A', inplace = True)
data2['right3'] = data3['right3']
data2['right3'] = data2['right3'].astype(float)
data2['plantID'] = data2['right3'] + data2['left']

data2['roottype'] = data3['Last']
tagcolumns1 = tagcolumns[:-1]
tagcolumns1.append('plantID')
dataC = data2[data2['roottype'] == 'C']
sumlats = dataC.groupby(tagcolumns1)['Total Length'].sum()
countlatroots = dataC.groupby(tagcolumns1)['Total Length'].count()
dataA = data2[data2['roottype'] == 'A']
sumPrimroots = dataA.groupby(tagcolumns1)['Total Length'].sum()
summary = pd.concat([sumlats, countlatroots, sumPrimroots], axis=1)
summary.columns = ['Total lateral root length (mm)', 'Number of lateral roots', 'Primary root length (mm)']
summary['ELRD'] = summary['Number of lateral roots'] / summary['Primary root length (mm)'].fillna(0)
summary.fillna(value = 0, inplace = True)
summary.reset_index(level=None, drop=False, inplace=True, col_level=0) #remove index to allow to build another

#summarise data by tags
N_count = summary.groupby(tagcolumns[:-1])['Primary root length (mm)'].count()
PrimMean = summary.groupby(tagcolumns[:-1])['Primary root length (mm)'].mean()
PrimStd = summary.groupby(tagcolumns[:-1])['Primary root length (mm)'].std()
NumlatsMean = summary.groupby(tagcolumns[:-1])['Number of lateral roots'].mean()
NumlatsStd = summary.groupby(tagcolumns[:-1])['Number of lateral roots'].std()
LatlenMean = summary.groupby(tagcolumns[:-1])['Total lateral root length (mm)'].mean()
LatlenStd = summary.groupby(tagcolumns[:-1])['Total lateral root length (mm)'].std()
LatdensMean = summary.groupby(tagcolumns[:-1])['ELRD'].mean()
LatdensStd = summary.groupby(tagcolumns[:-1])['ELRD'].std()
MeansDT = pd.concat([N_count, PrimMean, PrimStd, NumlatsMean, NumlatsStd, LatlenMean, LatlenStd, LatdensMean, LatdensStd], axis=1)
MeansDT.columns = ['N','Primary root length (mm)', 'Primary Length Std','Number of Lats', 'Number of Lats Std', 'Total Lat length','Total Lat length Std', 'Lateral Root Density','Lateral Root Density Std']
summary.loc[:,'Total root length (mm)']= summary.loc[:,'Primary root length (mm)'] + summary.loc[:,'Total lateral root length (mm)'] 

#Output to file
#asktosave = input ('Would you like to export .csv to directory? y/n:  ')
#if asktosave == 'y':
#    filenames = input('What prefix would you like to give to the .csv file?  ' )
save_path = csv_filepath.parents[0] / 'perplant.csv'
summary.to_csv(save_path)
print(".csv files have been written to directory")
    
#plot

p = PrimMean.plot('bar', yerr=PrimStd)
p.set_title("Mean root length", fontsize=18)
p.set_xlabel("Media", fontsize=18)
p.set_ylabel("Mean root length (mm)", fontsize=18)
    #p.set_ylim(0, 35)